@@ -38,13 +38,14 @@
 #include <libintl.h>
 
 #include <glib.h>
-#include <curl/curl.h>
+#include <glib/gstdio.h>
 
 #include <gwaei/definitions.h>
 #include <gwaei/regex.h>
 #include <gwaei/utilities.h>
 #include <gwaei/io.h>
 #include <gwaei/dictlist-object.h>
+#include <gwaei/preferences.h>
 
 
 static GwDictList *dictionaries;
@@ -177,7 +178,7 @@ GList* gw_dictlist_remove_first()
 //! The work of freeing each individual dictionary is automatically handled,
 //! removing the chance for mistakes.
 //!
-void gw_dictlist_free()
+void gw_dictlist_free ()
 {
     while (dictionaries->list != NULL)
       dictionaries->list = gw_dictlist_remove_first();
@@ -195,7 +196,7 @@ void gw_dictlist_free()
 //! @param request a const string to search for in the dictionary names
 //! @return returns the GwDictInfo dictionary object of the result or null.
 //!
-GwDictInfo* gw_dictlist_get_dictionary_by_name (const char* request)
+GwDictInfo* gw_dictlist_get_dictinfo_by_name (const char* request)
 {
     GList *current;
     current = dictionaries->list;
@@ -225,13 +226,13 @@ GwDictInfo* gw_dictlist_get_dictionary_by_name (const char* request)
 //! @param request a const string to search for in the dictionary names
 //! @return returns the GwDictInfo dictionary object of the result or null.
 //!
-GwDictInfo* gw_dictlist_get_dictionary_by_alias (const char* request)
+GwDictInfo* gw_dictlist_get_dictinfo_by_alias (const char* request)
 {
     if ((strcmp (request, "Radicals") == 0 || strcmp (request, "Kanji") == 0) &&
-         gw_dictlist_dictionary_get_status_by_id (GW_DICT_MIX) == GW_DICT_STATUS_INSTALLED )
-      return gw_dictlist_get_dictionary_by_name ("Mix");
+         gw_dictlist_dictionary_get_status_by_id (GW_DICT_ID_MIX) == GW_DICT_STATUS_INSTALLED )
+      return gw_dictlist_get_dictinfo_by_id (GW_DICT_ID_MIX);
     else 
-      return gw_dictlist_get_dictionary_by_name (request);
+      return gw_dictlist_get_dictinfo_by_name (request);
 }
 
 
@@ -245,7 +246,7 @@ GwDictInfo* gw_dictlist_get_dictionary_by_alias (const char* request)
 //! @param request a const string to search for in the dictionary names
 //! @return returns true if the dictionary is installed
 //!
-gboolean gw_dictlist_check_if_loaded_by_name(char* name)
+gboolean gw_dictlist_check_if_loaded_by_name (char* name)
 {
     GList *current = dictionaries->list;
     GwDictInfo *di;
@@ -273,7 +274,7 @@ gboolean gw_dictlist_check_if_loaded_by_name(char* name)
 //! @param status the integer status to check for
 //! @return returns the number of dictionaries with the status
 //!
-int gw_dictlist_get_total_with_status(int status)
+int gw_dictlist_get_total_with_status (GwDictStatus status)
 {
     GwDictInfo *di;
     GList *current = dictionaries->list;
@@ -303,7 +304,7 @@ int gw_dictlist_get_total_with_status(int status)
 //! @param id Id attribute of the wanted dictionary
 //! @return Returns the status of the dictionary
 //!
-int gw_dictlist_dictionary_get_status_by_id(int id)
+GwDictStatus gw_dictlist_dictionary_get_status_by_id (GwDictId id)
 {
     GList *current = dictionaries->list;
     GwDictInfo *di = (GwDictInfo*) current->data;
@@ -330,7 +331,7 @@ int gw_dictlist_dictionary_get_status_by_id(int id)
 //! @param id Id attribute of the wanted dictionary
 //! @return Returns a pointer to the dictionary if available
 //!
-GwDictInfo* gw_dictlist_get_dictionary_by_id(int id)
+GwDictInfo* gw_dictlist_get_dictinfo_by_id (GwDictId id)
 {
     GList *current = dictionaries->list;
     GwDictInfo *di = (GwDictInfo*) current->data;
@@ -382,12 +383,12 @@ GList* gw_dictlist_get_list()
 //! The built in dictionaries are set up, then any additional manually installed
 //! user dictionaries are searched for and set up.
 //!
-void gw_dictionaries_initialize_dictionary_list()
+void gw_dictionaries_initialize_dictionary_list ()
 {
     if (dictionaries != NULL)
       gw_dictlist_free ();
 
-    dictionaries = gw_dictlist_new();
+    dictionaries = gw_dictlist_new ();
        
     //Dictionaries having to do with gui elements
     gw_dictlist_add_dictionary ("English");
@@ -401,9 +402,11 @@ void gw_dictionaries_initialize_dictionary_list()
     gw_dictlist_add_dictionary ("German");
     gw_dictlist_add_dictionary ("Spanish");
 
+    if (gw_util_get_waei_directory () == NULL) return;
+
     //Path variables
     char path[FILENAME_MAX];
-    if (gw_util_get_waei_directory (path) == NULL) return;
+    strncpy(path, gw_util_get_waei_directory (), FILENAME_MAX);
     char *filename = &path[strlen (path)];
 
     //Directory variables
@@ -418,7 +421,7 @@ void gw_dictionaries_initialize_dictionary_list()
       {
         strcpy (filename, input);
         if (g_file_test(path, G_FILE_TEST_IS_REGULAR) == TRUE &&
-            regexec (&re_gz,  filename, 1, NULL, 0 ) != 0     &&
+            regexec (gw_re[GW_RE_FILENAME_GZ],  filename, 1, NULL, 0 ) != 0     &&
             strcmp  (filename, "English"           ) != 0     &&
             strcmp  (filename, "Kanji"             ) != 0     &&
             strcmp  (filename, "Names"             ) != 0     &&           
@@ -450,14 +453,14 @@ void gw_dictionaries_initialize_dictionary_list()
 static gboolean create_mix_dictionary()
 {
     GwDictInfo* mix;
-    mix = gw_dictlist_get_dictionary_by_id(GW_DICT_MIX);
+    mix = gw_dictlist_get_dictinfo_by_id(GW_DICT_ID_MIX);
     g_remove (mix->path);
     mix->status = GW_DICT_STATUS_NOT_INSTALLED;
 
     GwDictInfo* kanji;
-    kanji = gw_dictlist_get_dictionary_by_id(GW_DICT_KANJI);
+    kanji = gw_dictlist_get_dictinfo_by_id(GW_DICT_ID_KANJI);
     GwDictInfo* radicals;
-    radicals = gw_dictlist_get_dictionary_by_id(GW_DICT_RADICALS);
+    radicals = gw_dictlist_get_dictinfo_by_id(GW_DICT_ID_RADICALS);
 
     char *mpath = mix->path;
     char *kpath = kanji->path;
@@ -494,9 +497,9 @@ static gboolean create_mix_dictionary()
 static gboolean split_places_from_names_dictionary(GError **error)
 {
     GwDictInfo* di_places;
-    di_places = gw_dictlist_get_dictionary_by_id (GW_DICT_PLACES);
+    di_places = gw_dictlist_get_dictinfo_by_id (GW_DICT_ID_PLACES);
     GwDictInfo* di_names;
-    di_names = gw_dictlist_get_dictionary_by_id (GW_DICT_NAMES);
+    di_names = gw_dictlist_get_dictinfo_by_id (GW_DICT_ID_NAMES);
 
     if (di_names->status == GW_DICT_STATUS_NOT_INSTALLED) return FALSE;
 
@@ -555,41 +558,44 @@ static gboolean split_places_from_names_dictionary(GError **error)
 //! @param error set a GError to the pointer when an error occurs
 //! @returns Returns true on success
 //!
-void gw_dictlist_preform_postprocessing_by_name(char* name, GError **error)
+void gw_dictlist_preform_postprocessing_by_name (char* name, GError **error)
 {
     //Sanity check
     GwDictInfo* di;
-    di = gw_dictlist_get_dictionary_by_name (name);
+    di = gw_dictlist_get_dictinfo_by_name (name);
     if (di->status != GW_DICT_STATUS_INSTALLING &&
         di->status != GW_DICT_STATUS_UPDATING &&
         di->status != GW_DICT_STATUS_REBUILDING)
       return;
+    if (di->status == GW_DICT_STATUS_CANCELING)
+      return;
 
     //Setup some pointers
-    GwDictInfo* k_di = gw_dictlist_get_dictionary_by_id (GW_DICT_KANJI);
-    GwDictInfo* r_di = gw_dictlist_get_dictionary_by_id (GW_DICT_RADICALS);
-    GwDictInfo* n_di = gw_dictlist_get_dictionary_by_id (GW_DICT_NAMES);
-
-    //Preseve the status
-    int restore_status = di->status;
-    di->status = GW_DICT_STATUS_INSTALLED;
+    GwDictInfo* k_di = gw_dictlist_get_dictinfo_by_id (GW_DICT_ID_KANJI);
+    GwDictInfo* r_di = gw_dictlist_get_dictinfo_by_id (GW_DICT_ID_RADICALS);
+    GwDictInfo* n_di = gw_dictlist_get_dictinfo_by_id (GW_DICT_ID_NAMES);
 
     //Rebuild the mix dictionary
-    if ((di->id == GW_DICT_RADICALS || di->id == GW_DICT_KANJI || di->id == GW_DICT_MIX) &&
-        (k_di->status == GW_DICT_STATUS_INSTALLED && r_di->status == GW_DICT_STATUS_INSTALLED)
+    if ((di->id == GW_DICT_ID_RADICALS && k_di->status == GW_DICT_STATUS_INSTALLED) || 
+        (di->id == GW_DICT_ID_KANJI    && r_di->status == GW_DICT_STATUS_INSTALLED) ||
+        (di->id == GW_DICT_ID_MIX && k_di->status == GW_DICT_STATUS_INSTALLED && r_di->status == GW_DICT_STATUS_INSTALLED)
        )
     {
       di->status = GW_DICT_STATUS_REBUILDING;
       create_mix_dictionary ();
     }
     //Rebuild the names dictionary
-    else if(di->id == GW_DICT_NAMES && n_di->status == GW_DICT_STATUS_INSTALLED)
+    else if(di->id == GW_DICT_ID_NAMES)
     {
       di->status = GW_DICT_STATUS_REBUILDING;
       split_places_from_names_dictionary(error);
     }
-    //Cleanup
-    di->status = restore_status;
+
+    //Restore the previous state if the install wasn't canceled
+    if (di->status != GW_DICT_STATUS_CANCELING)
+    {
+      di->status = GW_DICT_STATUS_INSTALLED;
+    }
 }
 
 
@@ -606,7 +612,7 @@ void gw_dictlist_preform_postprocessing_by_name(char* name, GError **error)
 //! @param OLD const int of the current install status
 //! @param NEW const int of the new install status
 //!
-void gw_dictlist_normalize_all_status_from_to (const int OLD, const int NEW)
+void gw_dictlist_normalize_all_status_from_to (const GwDictStatus OLD, const GwDictStatus NEW)
 {
     GList *current = dictionaries->list;
     GwDictInfo *di;
@@ -643,6 +649,7 @@ void gw_dictlist_sync_dictionary (GwDictInfo *di, GError **error)
       return;
 
     printf("*  ");
+    // TRANSLATORS: The %s stands for the internal (untranslated) short name of the dictionary
     printf(gettext("Syncing %s dictionary"), di->name);
     printf("---------------------\n");
 
@@ -713,12 +720,12 @@ void gw_dictlist_update_dictionary_order_list ()
     i = 0;
     while (names[i] != NULL)
     {
-      di = gw_dictlist_get_dictionary_by_name (names[i]);
+      di = gw_dictlist_get_dictinfo_by_name (names[i]);
       if (di == NULL || di->status != GW_DICT_STATUS_INSTALLED)
         *names[i] = '\0';
-      if (di && di->id == GW_DICT_MIX) mix_name = names[i];
-      if (di && di->id == GW_DICT_KANJI) kanji_name = names[i];
-      if (di && di->id == GW_DICT_RADICALS) radicals_name = names[i];
+      if (di && di->id == GW_DICT_ID_MIX) mix_name = names[i];
+      if (di && di->id == GW_DICT_ID_KANJI) kanji_name = names[i];
+      if (di && di->id == GW_DICT_ID_RADICALS) radicals_name = names[i];
       i++;
     }
 
