@@ -1,4 +1,8 @@
+#ifndef GW_ENGINE_INCLUDED
+#define GW_ENGINE_INCLUDED
+
 //Starts a search.  Make sure to set target_text_buffer and target_text view
 void gw_search_get_results (GwSearchItem*);
 
 
+#endif
