@@ -36,14 +36,17 @@
 #include <libintl.h>
 
 #include <glib.h>
+#include <glib/gstdio.h>
 #include <curl/curl.h>
 
 #include <gwaei/definitions.h>
 #include <gwaei/regex.h>
+#include <gwaei/utilities.h>
 #include <gwaei/dictionary-objects.h>
+#include <gwaei/preferences.h>
 
 
-char     save_path[FILENAME_MAX]    = { '\0' };
+static char save_path[FILENAME_MAX] = { '\0' };
 
 
 //!
@@ -96,7 +99,7 @@ void gw_io_write_file(const char* write_mode, gchar *text)
 //!
 //! @return The status of the existance of rsync
 //!
-gboolean gw_io_check_for_rsync()
+gboolean gw_io_check_for_rsync ()
 {
     gboolean rsync_exists;
     rsync_exists = ( RSYNC != NULL     &&
@@ -124,6 +127,8 @@ gboolean gw_io_copy_with_encoding( char *source_path,     char *target_path,
                                   char *source_encoding, char *target_encoding,
                                   GError **error)
 {
+    if (*error != NULL) return FALSE;
+
     FILE* readfd = NULL;
     readfd = fopen (source_path, "r");
     if (readfd == NULL) exit(0);
@@ -145,13 +150,17 @@ gboolean gw_io_copy_with_encoding( char *source_path,     char *target_path,
     {
       inptr = buffer; outptr = output;
       inbytes_left = length; outbytes_left = length;
-      while (inbytes_left && outbytes_left && g_iconv (conv, &inptr, &inbytes_left, &outptr, &outbytes_left) != -1)
+      while (inbytes_left && outbytes_left)
       {
+        if (g_iconv (conv, &inptr, &inbytes_left, &outptr, &outbytes_left) == -1) break;
+
+        //Force increment if there is something wrong
         if (prev_inbytes == inbytes_left)
         {
           inptr++;
           inbytes_left--;
         }
+        //Normal operation
         prev_inbytes = inbytes_left;
         inptr = inptr + strlen(inptr) - inbytes_left;
         outptr = outptr + strlen(outptr) - outbytes_left;
@@ -160,46 +169,116 @@ gboolean gw_io_copy_with_encoding( char *source_path,     char *target_path,
     }
     g_iconv_close (conv);
 
-    close(readfd);
-    close(writefd);
+    fclose(readfd);
+    fclose(writefd);
 
     return TRUE;
 }
 
 
-static size_t write_func(void *ptr, size_t size, size_t nmemb, FILE *stream)
+//!
+//! \brief Private function made to be used with gw_io_download_file
+//!
+//! @param ptr TBA
+//! @param size TBA
+//! @param nmemb TBA
+//! @param stream TBA
+//!
+static size_t libcurl_write_func (void *ptr, size_t size, size_t nmemb, FILE *stream)
 {
     return fwrite(ptr, size, nmemb, stream);
 }
  
 
-static size_t read_func(void *ptr, size_t size, size_t nmemb, FILE *stream)
+//!
+//! \brief Private function made to be used with gw_io_download_file
+//!
+//! @param ptr TBA
+//! @param size TBA
+//! @param nmemb TBA
+//! @param stream TBA
+//!
+static size_t libcurl_read_func (void *ptr, size_t size, size_t nmemb, FILE *stream)
 {
-    return fread(ptr, size, nmemb, stream);
+    return fread (ptr, size, nmemb, stream);
 }
 
 
-gboolean gw_io_download_dictionary_file(char *source_path, char *save_path, gpointer func, gpointer data)
+//!
+//! \brief Private struct made to be used with gw_io_download_file
+//!
+//! @param ptr TBA
+//! @param size TBA
+//! @param nmemb TBA
+//! @param stream TBA
+//!
+typedef struct libcurl_callback_func_with_data {
+    int (*callback_function) (char*, int, gpointer);
+    gpointer data;
+} libcurl_callback_func_with_data; 
+
+
+//!
+//! \brief Private struct made to be used with gw_io_download_file
+//!
+//! @param ptr TBA
+//! @param size TBA
+//! @param nmemb TBA
+//! @param stream TBA
+//!
+int libcurl_update_progressbar (void   *data,
+                                double  dltotal,
+                                double  dlnow,
+                                double  ultotal,
+                                double  ulnow   )
 {
+    int percent = 0;
+    if (dltotal != 0.0)
+      percent = (int) (dlnow / dltotal * 100.0);
+    libcurl_callback_func_with_data *curldata = (libcurl_callback_func_with_data*) data;
+    return (curldata->callback_function) (NULL, percent, curldata->data);
+}
+
+
+//!
+//! \brief Downloads a file using libcurl
+//!
+//! @param source_path String of the source url
+//! @param save_path String of the path to save the file locally
+//! @param func Pointer to a function to update
+//! @param data gpointer to data to pass to the function pointer
+//! @param error Error handling
+//!
+gboolean gw_io_download_file (char *source_path, char *save_path,
+                              int (*callback_function) (char*, int, gpointer), gpointer data, GError **error)
+{
+    if (*error != NULL) return FALSE;
+
     CURL *curl;
     CURLcode res;
     FILE *outfile = NULL;
+    GwDictInfo *di = (GwDictInfo*) data;
 
-    curl = curl_easy_init();
+    curl = curl_easy_init ();
 
-    if(curl == NULL) return FALSE;
+    if (curl == NULL) return FALSE;
 
     outfile = fopen(save_path, "w");
 
     curl_easy_setopt(curl, CURLOPT_URL, source_path);
     curl_easy_setopt(curl, CURLOPT_WRITEDATA, outfile);
-    curl_easy_setopt(curl, CURLOPT_WRITEFUNCTION, write_func);
-    curl_easy_setopt(curl, CURLOPT_READFUNCTION, read_func);
-    if (func != NULL && data != NULL)
+    curl_easy_setopt(curl, CURLOPT_WRITEFUNCTION, libcurl_write_func);
+    curl_easy_setopt(curl, CURLOPT_READFUNCTION, libcurl_read_func);
+
+    libcurl_callback_func_with_data libcurl_data;
+    libcurl_data.callback_function = callback_function;
+    libcurl_data.data = data;
+
+    if (callback_function != NULL)
     {
       curl_easy_setopt(curl, CURLOPT_NOPROGRESS, 0L);
-      curl_easy_setopt(curl, CURLOPT_PROGRESSFUNCTION, func);
-      curl_easy_setopt(curl, CURLOPT_PROGRESSDATA, data);
+      curl_easy_setopt(curl, CURLOPT_PROGRESSFUNCTION, libcurl_update_progressbar);
+      curl_easy_setopt(curl, CURLOPT_PROGRESSDATA, &libcurl_data);
     }
 
     res = curl_easy_perform(curl);
@@ -207,59 +286,43 @@ gboolean gw_io_download_dictionary_file(char *source_path, char *save_path, gpoi
     fclose(outfile);
     curl_easy_cleanup(curl);
 
-    //If curl didn't write a clean file for some reason, end the function here
-    if (res != 0) g_remove(save_path);
+    if (res != 0)
+    {
+      g_remove(save_path);
+      if (di->status != GW_DICT_STATUS_CANCELING)
+      {
+        GQuark quark;
+        quark = g_quark_from_string (GW_GENERIC_ERROR);
+        const char *message = gettext(curl_easy_strerror(res));
+        if (error != NULL) *error = g_error_new_literal (quark, GW_FILE_ERROR, message);
+      }
+    }
 
     return (res == 0);
 }
 
 
-gboolean gw_io_delete_dictionary_file(GwDictInfo* file)
+//!
+//! \brief Copies a local file to another local location
+//!
+//! @param source_path String of the source url
+//! @param save_path String of the path to save the file locally
+//! @param error Error handling
+//!
+gboolean gw_io_copy_file (char *source_path, char *target_path, GError **error)
 {
-    printf("  ");
-    printf(gettext("Removing %s..."), file->long_name);
-    printf("\n");
+    if (*error != NULL) return FALSE;
 
-    printf("*   ");
-    printf(gettext("Removing %s..."), file->path);
-    if (g_remove(file->path) == 0)
-      printf("Success\n");
-    else
-      printf("Failed\n");
-
-    printf("*   ");
-    printf(gettext("Removing %s..."), file->gz_path);
-    if (g_remove(file->gz_path) == 0)
-      printf("Success\n");
-    else
-      printf("Failed\n");
-
-    printf("*   ");
-    printf(gettext("Removing %s..."), file->sync_path);
-    if (g_remove(file->sync_path) == 0)
-      printf("Success\n");
-    else
-      printf("Failed\n");
-
-    return TRUE;
-}
-
-
-gboolean gw_io_copy_dictionary_file(char *source_path, char *target_path, GError **error)
-{
     GQuark quark;
     quark = g_quark_from_string (GW_GENERIC_ERROR);
 
     char *contents = NULL;
-    gssize length;
+    gsize length;
     if ( g_file_get_contents(source_path, &contents, &length, NULL) == FALSE ||
          g_file_set_contents(target_path, contents, length, NULL) == FALSE     )
     {
-      printf("Error while copying the file...cleaning up.\n");
       remove(target_path);
-      const char *message = gettext("File copy error");
-      if (error != NULL)
-        *error = g_error_new_literal (quark, GW_FILE_ERROR, message);
+      if (error != NULL) *error = g_error_new_literal (quark, GW_FILE_ERROR, gettext("File copy error"));
       return FALSE;
     }
     if (contents != NULL) g_free(contents);
@@ -269,10 +332,8 @@ gboolean gw_io_copy_dictionary_file(char *source_path, char *target_path, GError
 
 
 //Creates a single dictionary containing both the radical dict and kanji dict
-gboolean gw_io_create_mix_dictionary(char *mpath, char *kpath, char *rpath)
+gboolean gw_io_create_mix_dictionary (char *mpath, char *kpath, char *rpath)
 {
-    printf(gettext("  Recreating mixed dictionary..."));
-
     FILE* kanji_file =  fopen(kpath, "r");
     FILE* output_file = fopen(mpath, "w");
     FILE *radicals_file = NULL;
@@ -355,8 +416,6 @@ gboolean gw_io_create_mix_dictionary(char *mpath, char *kpath, char *rpath)
     fclose(kanji_file);
     fclose(output_file);
 
-    printf(gettext("success\n"));
-
     return TRUE;
 }
 
@@ -468,8 +527,16 @@ gboolean gw_io_split_places_from_names_dictionary (char *spath, char *npath, cha
 }
 
 
-gboolean gw_io_gunzip_dictionary_file(char *path, GError **error)
+//!
+//! \brief Gunzips a file
+//!
+//! @param path String representing the path of the file to gunzip
+//! @param error Error handling
+//!
+gboolean gw_io_gunzip_file (char *path, GError **error)
 {
+    if (*error != NULL) return FALSE;
+
     GQuark quark;
     quark = g_quark_from_string (GW_GENERIC_ERROR);
     gboolean success;
@@ -484,15 +551,20 @@ gboolean gw_io_gunzip_dictionary_file(char *path, GError **error)
     if (success == FALSE) {
       g_remove(path);
       const char *message = gettext("gunzip error");
-      if (error != NULL)
-        *error = g_error_new_literal (quark, GW_FILE_ERROR, message);
+      if (error != NULL) *error = g_error_new_literal (quark, GW_FILE_ERROR, message);
     }
 
     return success;
 } 
 
 
-gboolean gw_io_unzip_dictionary_file(char *path, GError **error)
+//!
+//! \brief Unzips a file
+//!
+//! @param path String representing the path of the file to unzip
+//! @param error Error handling
+//!
+gboolean gw_io_unzip_file (char *path, GError **error)
 {
     GQuark quark;
     quark = g_quark_from_string (GW_GENERIC_ERROR);
@@ -515,13 +587,17 @@ gboolean gw_io_unzip_dictionary_file(char *path, GError **error)
     if (success == FALSE) {
       g_remove(path);
       const char *message = gettext("gunzip error");
-      if (error != NULL)
-        *error = g_error_new_literal (quark, GW_FILE_ERROR, message);
+      if (error != NULL) *error = g_error_new_literal (quark, GW_FILE_ERROR, message);
     }
     return success;
 }
 
 
+//!
+//! \brief Returns the total lines in a dictionary file for processing purposes.
+//!
+//! @param path The string representing path to the dictionary file
+//!
 int gw_io_get_total_lines_for_path (char *path)
 {
     //Calculate the number of lines in the dictionary
@@ -537,3 +613,170 @@ int gw_io_get_total_lines_for_path (char *path)
     return total_lines;
 }
 
+
+//!
+//! \brief Uninstalls a dictionary by it's name and all related dictionaries
+//!
+//! @param name The string representing the name of the dictionary to uninstall.
+//! @param callback_function The function to use to post status messages to.
+//! @param data gpointer to the data to pass to the callback_function
+//! @param long_messages Whether the messages should be long form or short form.
+//!
+void gw_io_uninstall_dictinfo (GwDictInfo *di,    int (*callback_function) (char*, int, gpointer),
+                                 gpointer data, gboolean long_messages                            )
+{
+    if (di == NULL) return;
+
+    char *message = NULL;
+    if (long_messages)
+      // TRANSLATORS: The %s stands for a file path
+      message = g_strdup_printf (gettext("Removing %s..."), di->path);
+    else
+      message = g_strdup_printf (gettext("Removing..."));
+
+    if (callback_function != NULL) {
+      callback_function (message, -1, data);
+    }
+
+    if (message != NULL)
+    {
+      g_free (message);
+      message = NULL;
+    }
+
+    g_remove(di->path);
+    g_remove(di->gz_path);
+    g_remove(di->sync_path);
+
+    di->status = GW_DICT_STATUS_NOT_INSTALLED;
+    di->load_position = -1;
+
+
+    GwDictInfo *r_di, *m_di, *k_di, *p_di;
+    r_di = gw_dictlist_get_dictinfo_by_id (GW_DICT_ID_RADICALS);
+    k_di = gw_dictlist_get_dictinfo_by_id (GW_DICT_ID_KANJI);
+    m_di = gw_dictlist_get_dictinfo_by_id (GW_DICT_ID_MIX);
+    p_di = gw_dictlist_get_dictinfo_by_id (GW_DICT_ID_PLACES);
+
+    if (di->id == GW_DICT_ID_KANJI)
+    {
+      gw_io_uninstall_dictinfo (r_di, callback_function, data, long_messages);
+      gw_io_uninstall_dictinfo (m_di, callback_function, data, long_messages);
+    }
+    else if (di->id == GW_DICT_ID_NAMES)
+    {
+      gw_io_uninstall_dictinfo (p_di, callback_function, data, long_messages);
+    }
+}
+
+
+//!
+//! \brief Installs a dictionary by it's name and all related dictionaries
+//!
+//! @param name The string representing the name of the dictionary to uninstall.
+//! @param callback_function The function to use to post status messages to.
+//! @param data gpointer to the data to pass to the callback_function
+//! @param long_messages Whether the messages should be long form or short form.
+//!
+void gw_io_install_dictinfo (GwDictInfo *di,    int (*callback_function) (char*, int, gpointer),
+                               gpointer data, gboolean long_messages, GError **error            )
+{
+    if (*error != NULL) return;
+
+    GQuark quark;
+    quark = g_quark_from_string (GW_GENERIC_ERROR);
+
+    if (di == NULL || di->status != GW_DICT_STATUS_NOT_INSTALLED) return;
+    di->status = GW_DICT_STATUS_INSTALLING;
+
+    char *message = NULL;
+
+    char fallback_uri[100];
+    gw_util_strncpy_fallback_from_key (fallback_uri, di->gckey, 100);
+
+    char uri[100];
+    gw_pref_get_string (uri, di->gckey, fallback_uri, 100);
+
+    if (long_messages == TRUE)
+    {
+      // TRANSLATORS: The %s stands for an URL
+      message = g_strdup_printf(gettext("Downloading %s..."), uri);
+      if (message != NULL)
+      {
+        callback_function (message, -1, NULL);
+        g_free (message);
+        message = NULL;
+      }
+    }
+
+    //Make sure the download folder exits
+    char *download_path = g_build_filename (gw_util_get_waei_directory(), "download", NULL);
+    if (download_path != NULL)
+    {
+      if (*error == NULL)
+      {
+        if ((g_mkdir_with_parents(download_path, 0755)) != 0 && di->status != GW_DICT_STATUS_CANCELING)
+        {
+          quark = g_quark_from_string (GW_GENERIC_ERROR);
+          *error = g_error_new_literal (quark, GW_FILE_ERROR, gettext("Unable to create dictionary folder"));
+        }
+      }
+      g_free (download_path);
+      download_path = NULL;
+    }
+
+    //Copy the file if it is a local file
+    if (*error == NULL && di->status != GW_DICT_STATUS_CANCELING)
+    {
+      if (g_file_test (uri, G_FILE_TEST_IS_REGULAR)) //Copy from local drive
+      {
+        gw_io_copy_file (uri, di->gz_path, error);
+      }
+      else //Download file over network
+      {
+        gw_io_download_file (uri, di->gz_path, callback_function, data, error);
+      }
+    }
+
+    //Decompress the file if necessary
+    if (*error == NULL && di->status != GW_DICT_STATUS_CANCELING && strstr(di->gz_path, ".gz"))
+    {
+      if (callback_function != NULL) {
+        callback_function (gettext("Decompressing..."), -1, data);
+      }
+      gw_io_gunzip_file (di->gz_path, error);
+    }
+    else if (*error == NULL && di->status != GW_DICT_STATUS_CANCELING && strstr(di->gz_path, ".zip"))
+    {
+      gw_io_unzip_file(di->gz_path, error);
+      if (*error == NULL && di->status != GW_DICT_STATUS_CANCELING)
+      {
+        quark = g_quark_from_string (GW_GENERIC_ERROR);
+        *error = g_error_new_literal (quark, GW_FILE_ERROR, gettext("Unzip Error"));
+      }
+    }
+
+    //Convert encoding if necessary
+    if (*error == NULL && di->status != GW_DICT_STATUS_CANCELING && strstr(di->sync_path, "UTF") == NULL)
+    {
+      if (callback_function != NULL) {
+        callback_function (gettext("Converting encoding..."), -1, data);
+      }
+      printf("converting encoding...\n");
+      gw_io_copy_with_encoding (di->sync_path, di->path, "EUC-JP","UTF-8", error);
+    }
+    else if (*error == NULL && di->status != GW_DICT_STATUS_CANCELING)
+    {
+      gw_io_copy_file (di->sync_path, di->path, error);
+    }
+
+    //Special dictionary post processing
+    if (*error == NULL && di->status != GW_DICT_STATUS_CANCELING)
+    {
+      if (callback_function != NULL) {
+        callback_function (gettext("Postprocessing..."), -1, data);
+      }
+      gw_dictlist_preform_postprocessing_by_name (di->name, error);
+    }
+}
+
