@@ -38,7 +38,8 @@
 #include <gwaei/definitions.h>
 #include <gwaei/utilities.h>
 
-static int run_mode;
+static GwRunmode run_mode;
+static char *gw_waei_folder_path = NULL;
 
 
 //!
@@ -48,22 +49,55 @@ static int run_mode;
 //! primarily interact with the user.  The two possible choices are GTK or
 //! the console.  QT is currently a red herring.
 //!
-//! @param call String describing the runmode to set.
+//! @param argc Standard main arguments
+//! @param argv Standard main arguments
 //!
-void gw_util_initialize_runmode (char* call)
+void gw_util_initialize_runmode (int argc, char *argv[])
 {
-   char *call_ptr = &call[strlen(call)];
-   while (call_ptr != call && !G_IS_DIR_SEPARATOR (*call_ptr))
+   char *call_ptr = argv[0];
+   call_ptr = &call_ptr[strlen(call_ptr)];
+   while (call_ptr != argv[0] && !G_IS_DIR_SEPARATOR (*call_ptr))
      call_ptr--;
    if (G_IS_DIR_SEPARATOR (*call_ptr))
      call_ptr++;
 
-   if (strcmp (call_ptr, "waei") == 0 || strcmp (INTERFACE, "NONE") == 0)
-     run_mode = GW_CONSOLE_RUNMODE;
+   gboolean ncurses_switch = FALSE;
+   //char *ptr = argv[0];
+   int i = 0;
+   while (i < argc && ncurses_switch == FALSE)
+   {
+     //WARNING this is a lazy implimentation...
+     ncurses_switch = (strcmp(argv[i], "-n") == 0 || strcmp(argv[i], "--ncurses") == 0);
+     i++;
+   }
+
+   if (strcmp (call_ptr, "waei") == 0)
+   {
+     if (ncurses_switch)
+     {
+       run_mode = GW_NCURSES_RUNMODE;
+     }
+     else
+     {
+       run_mode = GW_CONSOLE_RUNMODE;
+     }
+   }
+#ifdef WITH_GTK
    else if (strcmp (call_ptr, "gwaei") == 0)
+   {
      run_mode = GW_GTK_RUNMODE;
+   }
+#endif
+#ifdef WITH_QT
    else if (strcmp (call_ptr, "kwaei") == 0)
+   {
      run_mode = GW_QT_RUNMODE;
+   }
+#endif
+   else
+   {
+     run_mode = GW_CONSOLE_RUNMODE;
+   }
 }
 
 
@@ -76,107 +110,12 @@ void gw_util_initialize_runmode (char* call)
 //!
 //! @return Returns the current runmode as an integer
 //!
-int gw_util_get_runmode ()
+GwRunmode gw_util_get_runmode ()
 {
     return run_mode;
 }
 
 
-//!
-//! @brief Internal integer to string converson algorithm
-//!
-//! @param the input integer
-//! @param a preallocated char array to store the string
-//! @param the length of the array for the radix
-//! @return Returns true if there were no problems
-//!
-gboolean gw_util_itoa (int input, char *output, const int MAX)
-{
-    //digit_ptr is used to write and track the number
-    char buffer[MAX];
-    char *digit_ptr = &buffer[MAX - 1];
-
-    *digit_ptr = '\0';
-    //Write out the digits starting at the end of of the buffer
-    do
-    {
-      digit_ptr--;
-      *digit_ptr = input % 10;
-      *digit_ptr += 48;
-      input /= 10;
-    } while (input != 0 && digit_ptr != &output[0]);
-
-    //Shift the digits to the beginning of the string
-    strcpy(output, digit_ptr);
-
-    //Send return status
-    if (input != 0 && digit_ptr == &output[0])
-      return FALSE;
-    else
-      return TRUE;
-}
-
-//!
-//! @brief Converts a 2 digit hex number from a string to an integer
-//!
-//! @param the higher hex digit
-//! @param the lower hex digit
-//! @return Returns the int form of the hex number
-//!
-guint gw_util_2digithexstrtoint(char hex_char_digit_2, char hex_char_digit_1)
-{
-    int digit1;
-    if (hex_char_digit_1 <= '9')
-      digit1 = hex_char_digit_1 - '0';
-    else
-      digit1 = hex_char_digit_1 - 'A' + 10;
-
-    int digit2;
-    if (hex_char_digit_2 <= '9')
-      digit2 = hex_char_digit_2 - '0';
-    else 
-      digit2 = hex_char_digit_2 - 'A' + 10;
-
-    return ((digit2 << 12) | (digit1 << 8));
-}
-
-
-//!
-//! @brief Creates a hex string out of an integer
-//!
-//! @param a pre-allocated string to write into
-//! @param an integer representing a color
-//! @return Always returns true
-//!
-gboolean gw_util_itohexstr (char *color_string, guint color_integer)
-{
-    char *color_string_ptr = color_string;
-    *color_string_ptr = '#';
-    color_string_ptr++;
-    color_string_ptr = color_string_ptr + 6;
-    *color_string_ptr = '\0';
-    color_string_ptr--;
-
-    int color_integer_copy = color_integer;
-    int remainder = 0;
-
-    int i = 0;
-    while (i < 6)
-    {
-      remainder = color_integer_copy % 16;
-      color_integer_copy = color_integer_copy / 16;
-
-      if (remainder < 10)
-        *color_string_ptr = '0' + remainder;
-      else
-        *color_string_ptr = 'A' + (remainder - 10);
-      i++;
-      color_string_ptr--;
-    }
-
-    return TRUE;
-}
-
 //!
 //! @brief Gets the location of the waei dictionary folder
 //!
@@ -184,19 +123,22 @@ gboolean gw_util_itohexstr (char *color_string, guint color_integer)
 //! @param an integer representing a color
 //! @return Always returns true
 //!
-char* gw_util_get_waei_directory(char *buffer) 
+const char* gw_util_get_waei_directory() 
 {
-    const char* home = g_get_home_dir ();
-    char* directory = g_build_filename (home, ".waei", NULL);
-    strcpy (buffer, directory);
-    strcat (buffer, G_DIR_SEPARATOR_S);
-    g_free (directory);
+    if (gw_waei_folder_path != NULL)
+    {
+      g_free (gw_waei_folder_path);
+      gw_waei_folder_path = NULL;
+    }
 
-    //Make sure the ~/.waei directory exits, return NULL on error
-    if(g_mkdir_with_parents (buffer, 0755) == -1)
+    gw_waei_folder_path = g_build_filename (g_get_home_dir (), ".waei", NULL);
+
+    if(gw_waei_folder_path != NULL && g_mkdir_with_parents (gw_waei_folder_path, 0755) == -1)
+    {
       return NULL;
-    else
-      return buffer;
+    }
+    
+    return gw_waei_folder_path;
 }
 
 
@@ -234,7 +176,28 @@ gboolean gw_util_all_chars_are_in_range (char input[],
     {
       input_ptr = g_utf8_next_char(input_ptr);
       unic = g_utf8_get_char(input_ptr);
-      is_in_boundary = (unic >= start_unic_boundary && unic <= end_unic_boundary);
+      is_in_boundary = ((unic >= start_unic_boundary && unic <= end_unic_boundary) || unic == '.'
+                                                                                   || unic == '*'
+                                                                                   || (unic >= 0 && unic <= 9)
+                                                                                   || unic == '-'
+                                                                                   || unic == '^'
+                                                                                   || unic == '$'
+                                                                                   || unic == L'ー'
+                                                                                   || unic == '+'
+                                                                                   || unic == '?'
+                                                                                   || unic == '{'
+                                                                                   || unic == '/'
+                                                                                   || unic == '\\'
+                                                                                   || unic == '!'
+                                                                                   || unic == '{'
+                                                                                   || unic == '}'
+                                                                                   || unic == '['
+                                                                                   || unic == ']'
+                                                                                   || unic == '&'
+                                                                                   || unic == '|'
+                                                                                   || unic == ' '
+                                                                                   || unic == '('
+                                                                                   || unic == ')');
     }
 
     //Return Results
@@ -274,6 +237,48 @@ gboolean gw_util_is_katakana_str (char input[])
 }
 
 
+//!
+//! @brief Convenience function for seeing if a string is furigana
+//!
+//! Furigana are the characters containing both hiragana and katakana.
+//!
+//! @param input The string to check
+//! @return Returns true if it is in the range
+//! @see gw_util_all_chars_are_in_range ()
+//! @see gw_util_is_hiragana_str ()
+//! @see gw_util_is_kanji_str ()
+//! @see gw_util_is_romaji_str ()
+//!
+gboolean gw_util_is_furigana_str (char input[])
+{
+    return gw_util_all_chars_are_in_range (input, L'ぁ', L'ー');
+}
+
+
+
+//!
+//! @brief Convenience function for seeing if a string *starts* with kanji
+//!
+//! The point of the function is to find a word that starts with kanji but
+//! may also have hiragana in it somewheres.
+//!
+//! @param input The string to check
+//! @return Returns true if the function things this is a kanji string
+//! @see gw_util_all_chars_are_in_range ()
+//! @see gw_util_is_hiragana_str ()
+//! @see gw_util_is_katakana_str ()
+//! @see gw_util_is_romaji_str ()
+//!
+gboolean gw_util_is_kanji_ish_str (char input[])
+{
+    gboolean hira = gw_util_is_hiragana_str (input);
+    gboolean kata = gw_util_is_katakana_str (input);
+    gboolean roma = gw_util_is_romaji_str (input);
+    gboolean kanji = gw_util_is_kanji_str (input);
+
+    return (!hira && !kata && !roma && !kanji);
+}
+
 //!
 //! @brief Convenience function for seeing if a string is kanji
 //!
@@ -302,7 +307,7 @@ gboolean gw_util_is_kanji_str (char input[])
 //!
 gboolean gw_util_is_romaji_str (char input[])
 {
-    return gw_util_all_chars_are_in_range (input, L'A', L'z');
+    return gw_util_all_chars_are_in_range (input, L'A', L'ž');
 }
 
 
@@ -460,7 +465,7 @@ char* gw_util_next_hira_char_from_roma (char *input)
 //! @param input The string to write the hiragana equivalent to
 //! @return Returns null on error/end
 //!
-char* gw_util_roma_to_hira (char *input, char *output)
+char* gw_util_roma_char_to_hira (char *input, char *output)
 {
     //Set up the input pointer
     char *input_ptr;
@@ -841,6 +846,36 @@ char* gw_util_roma_to_hira (char *input, char *output)
 }
 
 
+//!
+//! @brief Convenience function to convert romaji to hiragana
+//!
+//! @param input The string to shift
+//! @see gw_util_shift_all_chars_in_str_by ()
+//! @see gw_util_str_shift_hira_to_kata ()
+//!
+gboolean gw_util_str_roma_to_hira (char* input, char* output, int max)
+{
+    //Try converting to hiragana
+    char *input_ptr = input;
+    char *kana_ptr = output;
+    *kana_ptr = '\0';
+    int leftover = max;
+    while (leftover-- > 0)
+    {
+      kana_ptr = gw_util_roma_char_to_hira (input_ptr, kana_ptr);
+      if (kana_ptr == NULL || input_ptr == NULL)
+        break;
+      input_ptr = gw_util_next_hira_char_from_roma (input_ptr);
+      if (kana_ptr == NULL || input_ptr == NULL)
+        break;
+
+      kana_ptr = &kana_ptr[strlen(kana_ptr)];
+    }
+
+    return (input_ptr != NULL && strlen (input_ptr) == 0);
+}
+
+
 //!
 //! @brief Checks for a Japanese ctype localization setting
 //!
@@ -861,6 +896,7 @@ gboolean gw_util_is_japanese_ctype ()
            );
 }
 
+
 //!
 //! @brief Checks for a Japanese local messages setting
 //!
@@ -885,6 +921,40 @@ gboolean gw_util_is_japanese_locale()
 }
 
 
+//!
+//! @brief Attempting to force a japanese CTYPE for better regex handling
+//!
+//! @return Returns whether the attempt was successful or not
+//!
+gboolean gw_util_force_japanese_locale ()
+{
+    //Try forcing a correct or better setting
+    //This will have nice effects like antialiased text
+    if ( setlocale(LC_CTYPE, "ja_JP.UTF8")  == NULL &&
+         setlocale(LC_CTYPE, "ja_JP.utf8")  == NULL &&
+         setlocale(LC_CTYPE, "ja_JP.UTF-8") == NULL &&
+         setlocale(LC_CTYPE, "ja_JP.utf-8") == NULL &&
+         setlocale(LC_CTYPE, "ja_JP")       == NULL &&
+         setlocale(LC_CTYPE, "ja")          == NULL &&
+         setlocale(LC_CTYPE, "japanese")    == NULL    )
+      //All failed, go for the default
+      setlocale(LC_CTYPE, "");
+
+    //printf("Set CTYPE locale to %s\n", setlocale(LC_CTYPE, NULL));
+
+    if ( setlocale(LC_COLLATE, "ja_JP.UTF8")  == NULL &&
+         setlocale(LC_COLLATE, "ja_JP.utf8")  == NULL &&
+         setlocale(LC_COLLATE, "ja_JP.UTF-8") == NULL &&
+         setlocale(LC_COLLATE, "ja_JP.utf-8") == NULL &&
+         setlocale(LC_COLLATE, "ja_JP")       == NULL &&
+         setlocale(LC_COLLATE, "ja")          == NULL &&
+         setlocale(LC_COLLATE, "japanese")    == NULL    )
+      //All failed, go for the default
+      setlocale(LC_COLLATE, "");
+}
+
+
+
 //!
 //! @brief A function that gets the default pref hardwired in.
 //!
@@ -896,7 +966,7 @@ gboolean gw_util_is_japanese_locale()
 //! @param key the key of the wanted preference
 //! @param n the radix of the size of the value array
 //!
-void gw_util_strncpy_fallback_from_key (char *value, char *key, int n)
+void gw_util_strncpy_fallback_from_key (char *value, const char *key, int n)
 {
     if (strcmp (GCKEY_GW_ENGLISH_SOURCE, key) == 0)
       strncpy (value, GW_ENGLISH_URI_FALLBACK, n);
@@ -923,4 +993,92 @@ void gw_util_strncpy_fallback_from_key (char *value, char *key, int n)
 }
 
 
+//!
+//! @brief Creates an allocated string from the arguments passed to the program
+//!
+//! @param argc Standard main argument
+//! @param argv Standard main argument
+//!
+//! @return Returns an allocated string containing the query text.  Should be freed with g_free ()
+//!
+char *gw_util_strdup_args_to_query (int argc, char *argv[])
+{
+      int i = 1;
+      char *query_text_data = NULL;
+      char *query_temp = NULL;
+      //query_text_data = g_strdup_printf ("");
+      while (i < argc)
+      {
+        //First loop
+        if (query_text_data == NULL)
+        {
+          query_text_data = g_strdup_printf("%s", argv[i]);
+        }
+        //Next loops
+        else
+        {
+          query_temp = g_strdup_printf("%s %s", query_text_data, argv[i]);
+          if (query_text_data != NULL)
+          {
+            g_free (query_text_data);
+            query_text_data = NULL;
+          }
+          query_text_data = query_temp;
+          query_temp = NULL;
+        }
+        i++;
+      }
+      if (query_temp != NULL)
+      {
+        g_free (query_temp);
+        query_temp = NULL;
+      }
 
+      return query_text_data;
+}
+
+//!
+//! @brief Sanitize an input string
+//!
+//! This function will check if the input string is a valid utf-8 sequence,
+//! it will then normalize this string in the Normalization Form Canonical Composition,
+//! then replace the bytes of unprintable unicode glyphe (like control codepoint) with spaces,
+//! and finally will remove leading and trailing spaces if asked to.
+//!
+//! @param text an utf8 encoded string to sanitize
+//! @param strip if true remove leading and trailing spaces
+//! @return a newly allocated sanitized utf8 encoded string or NULL if text was too.
+//!         If the result is non-NULL it must be freed with g_free(). 
+//!
+char* gw_util_sanitize_input(char *text, gboolean strip)
+{
+  if(text == NULL)
+    return NULL;
+  
+  // First validate the utf8 input data
+  char *end; // pointer to the valid end of utf8, it is before or at the end of *text 
+  if (!g_utf8_validate(text, -1, (const char **) &end))
+    *end = '\0'; // uh oh, was not valid utf8, let's put a stop at the last valid position
+    
+  // Then let's normalize utf8 : there can be several encodings for same glyph, 
+  // let's always use the same for the sake of consistency
+  // see http://library.gnome.org/devel/glib/stable/glib-Unicode-Manipulation.html#g-utf8-normalize
+  // and http://en.wikipedia.org/wiki/Unicode_normalization
+  char *ntext = g_utf8_normalize (text, -1,  G_NORMALIZE_NFC ); // this allocate a new char*
+  
+  // for each unicode symbol replace unprintable symbol bytes with spaces  
+  char *ptr = ntext;  // pointer to the start of current glyph
+  char *next = NULL;  // pointer to the start of original next glyph
+  while (*ptr != '\0') 
+  {
+    next = g_utf8_next_char (ptr);
+    if (!g_unichar_isprint (g_utf8_get_char (ptr)) )
+      strncpy(ptr, "         ", next - ptr);
+    ptr = next;
+  }
+
+  if(strip)    
+    g_strstrip(ntext); // no new allocation, just modifying the string
+   
+  return ntext;
+}
